
#include <libgnomeui/gnome-app.h>
#include <libgnomeui/gnome-init.h>
#include <libgnomeui/gnome-app-helper.h>
#include <gnome.h>

extern "C" {
#include <gtk/gtk.h>
#include <glib.h>
}

#include <pthread.h>

#include "guis/gnome/decoder_gnome.hh"
#include "version.hh"

#include <unistd.h>
#include <iostream.h>
using namespace std;

static GnomeApp* app;
GtkWidget* drawingarea;

static GtkWidget *file_selector;
static gchar*    default_dir = NULL;

static pthread_t decoder_thread;
static bool decoder_running = false;


gint idle_decode(gpointer data)
{
  if (Gnome_ViewMPEG())
    return TRUE;

  decoder_running=false;
  return FALSE;
}


void store_filename(GtkFileSelection *selector, gpointer user_data)
{
  gchar *selected_filename;
  selected_filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_selector));
  printf("FILENAME: >%s<\n",selected_filename);

  if (default_dir) g_free(default_dir);
  gchar* dir = g_dirname(selected_filename);
  default_dir = g_strconcat(dir,"/",NULL);
  g_free(dir);

  if (decoder_running)
    {
      cout << "another decoder is running\n";
    }
  else
    {
      if (Gnome_LoadMPEG(selected_filename))
	{
	  decoder_running=true;
	  //pthread_create(&decoder_thread,NULL,DecoderThread,NULL);
	  //DecoderThread(NULL);
	  gtk_idle_add(idle_decode,NULL);
	}
    }
}

static gint 
open_cb(GtkWidget* window, GdkEventAny* e, gpointer data)
{
  /* Create the selector */
    
  file_selector = gtk_file_selection_new("Open MPEG file...");
    
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
		      "clicked", GTK_SIGNAL_FUNC (store_filename), NULL);
                            
  /* Ensure that the dialog box is destroyed when the user clicks a button. */
    
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT(file_selector));

  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->cancel_button),
			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT(file_selector));

  // initialize

  if (default_dir) gtk_file_selection_set_filename(GTK_FILE_SELECTION(file_selector),
						   default_dir);
    
  /* Display that dialog */
    
  gtk_widget_show (file_selector);

  return FALSE;
}

static gint 
about_cb(GtkWidget* window, gpointer data)
{
  const gchar *authors[] = {
    "Dirk Farin",
      NULL
      };

  gtk_widget_show (gnome_about_new ("DVDview", DVDVIEW_VERSION,
				    "Copyright 2001 Dirk Farin",
				    (const gchar **) authors,
				    ("DVDview is a software only MPEG-1 and MPEG-2 decoder."),
				    NULL));

  return FALSE;
}

static gint 
exit_cb(GtkWidget* window, GdkEventAny* e, gpointer data)
{
  gtk_main_quit();
  return FALSE;
}

static gint 
delete_event_cb(GtkWidget* window, GdkEventAny* e, gpointer data)
{
  gtk_main_quit();
  return FALSE;
}

gint exposure_cb( GtkWidget *widget,
		  GdkEvent  *event,
		  gpointer   callback_data )
{
  Gnome_Redraw();
  return FALSE;
}


static GnomeUIInfo file_menu[] = {
  GNOMEUIINFO_MENU_OPEN_ITEM(open_cb,NULL),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_EXIT_ITEM(exit_cb,NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo help_menu [] = {
  //GNOMEUIINFO_HELP ("gnome-hello"),
   
   GNOMEUIINFO_MENU_ABOUT_ITEM(about_cb, NULL),
   GNOMEUIINFO_END
};

static GnomeUIInfo main_menu[] = {
  GNOMEUIINFO_SUBTREE("_File", file_menu),
  GNOMEUIINFO_SUBTREE("_Help", help_menu),
  GNOMEUIINFO_END
};

void start_GUI_gnome(int argc,char** argv)
{
  //g_thread_init(NULL);
  gnome_init("dvdview",DVDVIEW_VERSION,argc,argv);
  gdk_rgb_init();

  app = GNOME_APP(gnome_app_new("dvdview",
				" DVDview " DVDVIEW_VERSION "    " DVDVIEW_DATE " (c) Dirk Farin"));
  gnome_app_create_menus(app, main_menu);
  gtk_window_set_policy(GTK_WINDOW(app), FALSE, FALSE, TRUE);

  GtkWidget* vbox = gtk_vbox_new(FALSE,2);
  drawingarea = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(drawingarea),320,240);

  gtk_box_pack_start(GTK_BOX(vbox),drawingarea,FALSE,FALSE,0);
  //gtk_box_pack_start(GTK_BOX(vbox),gtk_hseparator_new(),FALSE,FALSE,0);

  //GtkWidget* drawingarea2 = gtk_drawing_area_new();
  //gtk_drawing_area_size(GTK_DRAWING_AREA(drawingarea2),320,240);

  //gtk_box_pack_start(GTK_BOX(vbox),drawingarea2,FALSE,FALSE,0);

  gtk_signal_connect(GTK_OBJECT(app),
		     "delete_event",
		     GTK_SIGNAL_FUNC(delete_event_cb),
		     NULL);

  gtk_signal_connect(GTK_OBJECT(drawingarea),
		     "expose_event",
		     GTK_SIGNAL_FUNC(exposure_cb),
		     NULL);

  gnome_app_set_contents(app,vbox);
  gtk_widget_show_all(GTK_WIDGET(app));


  // initialize decoder

  Gnome_DecoderInit();

  gtk_main();

  // GtkTable* table = gtk_new_table(2,
}

